<?php


namespace Schaffen\Core\Person\Managers;


use Freelabois\LaravelQuickstart\Extendables\ManipulationManager;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Schaffen\Core\Person\Models\Address;
use Schaffen\Core\Person\Models\Person;
use Schaffen\Core\Person\Repositories\ContactRepository;
use Schaffen\Core\Person\Repositories\PersonModalityRepository;
use Schaffen\Core\Person\Repositories\PersonRepository;
use Schaffen\Core\User\Repositories\UserRepository;
use Schaffen\Financial\BillingRoute\Repositories\ClientRouteRepository;
use Schaffen\Core\Person\Repositories\AddressRepository;


class PersonManager extends ManipulationManager
{
    protected $validation = [
        self::CREATE => [],
        self::EDIT => []
    ];

    /**
     * @var PersonRepository
     */
    protected $repository;
    /**
     * @var ClientRouteRepository
     */
    protected $client_route_repository;
    /**
     * @var UserRepository
     */
    protected $user_repository;
    /**
     * @var AddressRepository
     */
    protected $address_repository;
    /**
     * @var ContactRepository
     */
    protected $contact_repository;


    public function __construct(
        PersonRepository      $repository,
        ClientRouteRepository $client_route_repository,
        UserRepository        $user_repository,
        AddressRepository     $address_repository,
        ContactRepository     $contact_repository
    )
    {
        $this->repository = $repository;
        $this->client_route_repository = $client_route_repository;
        $this->user_repository = $user_repository;
        $this->address_repository = $address_repository;
        $this->contact_repository = $contact_repository;
    }

    /**
     * @param $values
     * @param int|null $id
     * @param array $relations
     * @return Person
     * @throws \Throwable
     */
    public function storeOrUpdate($values, int $id = null, $relations = []): Person
    {
        return DB::transaction(function () use ($values, $id, $relations) {
            /** @var Person $person */
            $person = parent::storeOrUpdate($values, $id, $relations);
            if ($id === null) {
                /** @var AddressManager $addressManager */
                $addressManager = app()->make(AddressManager::class);
                $address = $values['address'];
                $address['pee_padrao'] = 1;
                $address['pee_fk_pes_codigo'] = $person->pes_codigo;
                $addressManager->storeOrUpdate($address);

                /** @var ContactManager $contactManager */
                $contactManager = app()->make(ContactManager::class);
                $contact = $values['contact'];
                $contact['peco_contato'] = $person->pes_nome;
                $contact['peco_padrao'] = 1;
                $contact['peco_fk_pes_codigo'] = $person->pes_codigo;
                $contactManager->storeOrUpdate($contact);
                $person->load(['address', 'contact']);
            } else {
                $address = $this->address_repository->first(['pee_fk_pes_codigo' => $id]);
                if ($address) {
                    $this->address_repository->storeOrUpdate($values['address'], $address->pee_codigo);
                } else {
                    $address = $values['address'];
                    $address['pee_padrao'] = 1;
                    $address['pee_fk_pes_codigo'] = $person->pes_codigo;
//                    dd($address);
                    $this->address_repository->storeOrUpdate($address);
                }


                $contact = $this->contact_repository->first(['peco_fk_pes_codigo' => $id]);
                if ($contact) {
                    $this->contact_repository->storeOrUpdate($values['contact'], $contact->peco_codigo);
                } else {
                    $contact = $values['contact'];
                    $contact['peco_contato'] = $person->pes_nome;
                    $contact['peco_fk_pes_codigo'] = $person->pes_codigo;
                    $contact['peco_padrao'] = 1;
                    dd($contact);
                    $this->contact_repository->storeOrUpdate($contact);
                }
            }
            return $person;
        });
    }

    public function sincronization($values)
    {
//        dd($values);
        foreach ($values as $person) {
            $person["pes_identificador"] = $this->limpaCPF_CNPJ($person["pes_identificador"]);
            $rota_cliente = [];
            $pessoa = $this->repository->first(['pes_identificador' => $person['pes_identificador']], ['rota_que_pertence_cliente']);
//            dd($pessoa);
            if (!$pessoa) {
                $pessoa = $this->storeOrUpdate($person);
            } else {
                $pessoa = $this->storeOrUpdate($person, $pessoa->pes_codigo);
            }

            $rota_cliente['roc_fk_pes_codigo'] = $pessoa->pes_codigo;

            $client_route = $this->client_route_repository->first(['roc_fk_pes_codigo' => $pessoa->pes_codigo]);

            if ($person['rot_codigo']) {
                $rota_cliente['roc_fk_rot_codigo'] = $person['rot_codigo'];
            } else {
                if ($client_route) {
                    $rota_cliente['roc_fk_rot_codigo'] = $client_route['roc_fk_rot_codigo'];
                    $roc_cod = $client_route->roc_codigo;
                } else {
                    $t = $this->user_repository->first(['usu_codigo' => auth()->user()['usu_codigo']], ['person']);
                    $rota_cliente['roc_fk_rot_codigo'] = $t->person->route[0]->rot_codigo;
                    $roc_cod = null;
                }
            }

            if ($client_route) {
                $rota_cliente = $this->client_route_repository->storeOrUpdate($rota_cliente, $client_route->roc_codigo);
            } else {
                $rota_cliente = $this->client_route_repository->storeOrUpdate($rota_cliente);
            }
            $this->setPessoaModalidade($rota_cliente->roc_fk_pes_codigo);

        }
        return $this->clientesSincronizados($values);
    }

    public function clientesSincronizados($values)
    {
        $clientes = [];
        foreach ($values as $value) {
            $clientes[] = $this->repository->first(['pes_identificador' => $value['pes_identificador']], ['address', 'contact']);
        }
        return $clientes;
    }

    public function setPessoaModalidade($id_cliente)
    {
        $user_logado = auth()->user()['usu_nome'];
        $pessoa_modalidade_repository = app()->make(PersonModalityRepository::class);

        $cliente = $pessoa_modalidade_repository->first(['pem_fk_pes_codigo' => $id_cliente]);

        if (!$cliente) {
            $user_insert = $user_logado;

        } else {
            $user_insert = $cliente->pem_user_insert;
            $user_update = $user_logado;
        }


        $values = [
            "pem_fk_pes_codigo" => $id_cliente,
            "pem_fk_mdd_codigo" => 3, // 3 é o código de cliente na tabela pessoa_modalidade
            "pem_status" => 1,
            "pem_user_insert" => $user_insert,
            "pem_user_update" => $user_update ?? "",
        ];
        if ($cliente) {
            $pessoa_modalidade_repository->storeOrUpdate($values, $id_cliente);
        } else {
            $pessoa_modalidade_repository->storeOrUpdate($values);
        }


        return true;
    }


    function limpaCPF_CNPJ($valor)
    {
        $valor = trim($valor);
        $valor = str_replace(".", "", $valor);
        $valor = str_replace(",", "", $valor);
        $valor = str_replace("-", "", $valor);
        $valor = str_replace("/", "", $valor);
        return $valor;
    }

}
