<?php
/**
 * Created for Schaffen IT.
 * User: Thiago Traczykowski
 */

namespace Schaffen\Core\Product\Models;

use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Schaffen\Financial\Price\Models\ProductPrice;
use Schaffen\ProductionOrder\ProductionOrder\Models\CompositeProduct;

class ProductVarietyPack extends Model
{
    const CREATED_AT = "pve_date_insert";
    const CREATED_BY = "pve_user_insert";
    const UPDATED_AT = "pve_date_update";
    const UPDATED_BY = "pve_user_update";
    protected $connection = 'dynamicdb';
    protected $table = "produto.produto_variedade_embalagem";
    protected $primaryKey = "pve_codigo";
    protected $fillable = [
        "pve_fk_prv_codigo",
        "pve_fk_unm_codigo",
        "pve_quantidade_produto",
        "pve_variacao_compra",
        "pve_variacao_venda",
        "pve_desconto_maximo",
        "pve_padrao",
        "pve_ordem",
        "pve_estoque_minimo",
        "pve_estoque_maximo",
        "pve_date_insert",
        "pve_user_insert",
        "pve_date_update",
        "pve_user_update",
        "pve_codigo_barras",
        "pve_fk_unm_codigo_trib",
        "pve_quantidade_trib",
        "pve_controla_estoque",
        "pve_quantidade_desconto_condicional",
        "pve_cean",
        "pve_cean_trib",
        "pve_ignorar_cean",
        "pve_codigo_referencia",
        "pve_codigo_auxiliar",
        "pve_base"
    ];
    protected $casts = [
        'pve_fk_unm_codigo' => 'string',
        'pve_fk_unm_codigo_trib' => 'string',
    ];
    protected $searchableFields = [
        ["field" => 'pve_fk_prv_codigo'],

    ];

    protected static function boot()
    {
        parent::boot();

        static::addGlobalScope('product-name', function (Builder $query) {
            $field = "pro_descricao||' - '||prv_descricao||' - '||unm_descricao";
            $field .= " || case when trim(pve_codigo_barras) <> '' then ' - ('||pve_codigo_barras||')' else '' end";
            $field .= " || case when mar_codigo <> '-1' then' - '||mar_descricao||'' else '' end";
            $query->select(["produto.produto_variedade_embalagem.*", DB::raw("($field) as product_complete_name")]);
            $query->join('produto.produto_variedade', function ($q) {
                return $q->on('pve_fk_prv_codigo', 'prv_codigo');
            });
            $query->join('produto.produto', function ($q) {
                return $q->on('pro_codigo', 'prv_fk_pro_codigo');
            });
            $query->join('cadastro.unidade_medida', function ($q) {
                return $q->on('unm_codigo', 'pve_fk_unm_codigo');
            });
            $query->join('produto.marca', function ($q) {
                return $q->on('mar_codigo', 'pro_fk_mar_codigo');
            });

            return $query;
        });
    }

    public function measurement_unit()
    {
        return $this->belongsTo(MeasurementUnit::class, 'pve_fk_unm_codigo');
    }

    public function product_variety()
    {
        return $this->belongsTo(ProductVariety::class, 'pve_fk_prv_codigo')->with('product_mobile');
    }

    public function composition()
    {
        return $this->hasMany(CompositeProduct::class, 'pcp_fk_pve_codigo_materia_prima');
    }

    public function price(){
        return $this->hasMany(ProductPrice::class, 'tpp_fk_pve_codigo');
    }
}
